/*
 * The contents of this file are subject to the terms 
 * of the Common Development and Distribution License 
 * (the "License").  You may not use this file except 
 * in compliance with the License.
 * 
 * You can obtain a copy of the license at 
 * glassfish/bootstrap/legal/CDDLv1.0.txt or 
 * https://glassfish.dev.java.net/public/CDDLv1.0.html. 
 * See the License for the specific language governing 
 * permissions and limitations under the License.
 * 
 * When distributing Covered Code, include this CDDL 
 * HEADER in each file and include the License file at 
 * glassfish/bootstrap/legal/CDDLv1.0.txt.  If applicable, 
 * add the following below this CDDL HEADER, with the 
 * fields enclosed by brackets "[]" replaced with your 
 * own identifying information: Portions Copyright [yyyy] 
 * [name of copyright owner]
 */
package oracle.toplink.essentials.testing.tests.cmp3.inheritance;

import oracle.toplink.essentials.testing.framework.junit.JUnitTestCase;
import oracle.toplink.essentials.testing.models.cmp3.inheritance.InheritanceTableCreator;
import oracle.toplink.essentials.testing.models.cmp3.inheritance.Engineer;
import oracle.toplink.essentials.testing.models.cmp3.inheritance.Laptop;
import oracle.toplink.essentials.testing.models.cmp3.inheritance.ComputerPK;
import junit.framework.Test;
import junit.framework.TestSuite;
import junit.extensions.TestSetup;

import javax.persistence.EntityManager;

/**
 * Inheritance test suite.
 */
public class InheritanceJUnitTest extends JUnitTestCase {

    public InheritanceJUnitTest() {
    }

    public InheritanceJUnitTest(String name) {
        super(name);
    }

    public static Test suite() {
        TestSuite suite = new TestSuite(InheritanceJUnitTest.class);

        return new TestSetup(suite) {
        
            protected void setUp() {
                new InheritanceTableCreator().replaceTables(JUnitTestCase.getServerSession());
            }

            protected void tearDown() {
                clearCache();
            }
        };
    }
    
    // JUnit framework will automatically execute all methods starting with test...

    // test GF#1153 failure to associate a child entity which has JOINED strategy and EmbeddedId
    public void testAssociationWithJoinedAndEmbeddedIdChildEntity() {
        EntityManager em = createEntityManager();
        em.getTransaction().begin();

        Engineer engineer = new Engineer();
        ComputerPK computerPK = new ComputerPK("Dell", "100-001");
        Laptop laptop;
        try {
            em.persist(engineer);
            laptop = em.find(Laptop.class, computerPK);
            if(laptop == null){
                laptop = new Laptop(computerPK);
                em.persist(laptop);
            }
            
            // associate many-to-many relationships
            engineer.getLaptops().add(laptop);
            
            em.getTransaction().commit();
        } catch(RuntimeException ex){
            if(em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            throw ex;
        } finally {
            em.close();
        }
    }
}
